@@ -312,6 +312,30 @@ empathy_account_chooser_new (void)
 	return chooser;
 }
 
+gboolean
+empathy_account_chooser_has_all_selected (EmpathyAccountChooser *chooser)
+{
+	EmpathyAccountChooserPriv *priv;
+	GtkTreeModel              *model;
+	GtkTreeIter                iter;
+	RowType                    type;
+
+	g_return_val_if_fail (EMPATHY_IS_ACCOUNT_CHOOSER (chooser), FALSE);
+
+	priv = GET_PRIV (chooser);
+
+	g_return_val_if_fail (priv->has_all_option == TRUE, FALSE);
+
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (chooser));
+	if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (chooser), &iter)) {
+		return FALSE;
+	}
+
+	gtk_tree_model_get (model, &iter, COL_ACCOUNT_ROW_TYPE, &type, -1);
+
+	return type == ROW_ALL;
+}
+
 /**
  * empathy_account_chooser_dup_account:
  * @chooser: an #EmpathyAccountChooser
@@ -414,6 +438,30 @@ empathy_account_chooser_set_account (EmpathyAccountChooser *chooser,
 	return data.set;
 }
 
+void
+empathy_account_chooser_set_all (EmpathyAccountChooser *chooser)
+{
+	EmpathyAccountChooserPriv *priv;
+	GtkComboBox    *combobox;
+	GtkTreeModel   *model;
+	GtkTreeIter     iter;
+
+	g_return_if_fail (EMPATHY_IS_ACCOUNT_CHOOSER (chooser));
+
+	priv = GET_PRIV (chooser);
+
+	g_return_if_fail (priv->has_all_option);
+
+	combobox = GTK_COMBO_BOX (chooser);
+	model = gtk_combo_box_get_model (combobox);
+
+	if (gtk_tree_model_get_iter_first (model, &iter)) {
+		/* 'All accounts' is the first row */
+		gtk_combo_box_set_active_iter (combobox, &iter);
+		priv->account_manually_set = TRUE;
+	}
+}
+
 /**
  * empathy_account_chooser_get_has_all_option:
  * @chooser: an #EmpathyAccountChooser
@@ -488,7 +536,7 @@ empathy_account_chooser_set_has_all_option (EmpathyAccountChooser *chooser,
 
 		gtk_list_store_prepend (store, &iter);
 		gtk_list_store_set (store, &iter,
-				    COL_ACCOUNT_TEXT, _("All"),
+				    COL_ACCOUNT_TEXT, _("All accounts"),
 				    COL_ACCOUNT_ENABLED, TRUE,
 				    COL_ACCOUNT_POINTER, NULL,
 				    COL_ACCOUNT_ROW_TYPE, ROW_ALL,
@@ -609,7 +657,7 @@ account_chooser_setup (EmpathyAccountChooser *chooser)
 	gtk_cell_layout_clear (GTK_CELL_LAYOUT (combobox));
 
 	store = gtk_list_store_new (COL_ACCOUNT_COUNT,
-				    G_TYPE_STRING,    /* Image */
+				    GDK_TYPE_PIXBUF,    /* Image */
 				    G_TYPE_STRING,    /* Name */
 				    G_TYPE_BOOLEAN,   /* Enabled */
 				    G_TYPE_UINT,      /* Row type */
@@ -625,10 +673,9 @@ account_chooser_setup (EmpathyAccountChooser *chooser)
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combobox), renderer, FALSE);
 	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combobox), renderer,
-					"icon-name", COL_ACCOUNT_IMAGE,
+					"pixbuf", COL_ACCOUNT_IMAGE,
 					"sensitive", COL_ACCOUNT_ENABLED,
 					NULL);
-	g_object_set (renderer, "stock-size", GTK_ICON_SIZE_BUTTON, NULL);
 
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combobox), renderer, TRUE);
@@ -698,8 +745,15 @@ account_chooser_find_account_foreach (GtkTreeModel *model,
 {
 	FindAccountData *data = user_data;
 	TpAccount  *account;
+	RowType type;
 
-	gtk_tree_model_get (model, iter, COL_ACCOUNT_POINTER, &account, -1);
+	gtk_tree_model_get (model, iter,
+		COL_ACCOUNT_POINTER, &account,
+		COL_ACCOUNT_ROW_TYPE, &type,
+		 -1);
+
+	if (type != ROW_ACCOUNT)
+		return FALSE;
 
 	if (account == data->account) {
 		data->found = TRUE;
@@ -763,6 +817,7 @@ account_chooser_filter_ready_cb (gboolean is_enabled,
 	GtkListStore              *store;
 	GtkComboBox               *combobox;
 	const gchar               *icon_name;
+	GdkPixbuf                 *pixbuf;
 
 	chooser = fr_data->chooser;
 	priv = GET_PRIV (chooser);
@@ -772,13 +827,18 @@ account_chooser_filter_ready_cb (gboolean is_enabled,
 	store = GTK_LIST_STORE (gtk_combo_box_get_model (combobox));
 
 	icon_name = tp_account_get_icon_name (account);
+	pixbuf = empathy_pixbuf_from_icon_name (icon_name,
+		GTK_ICON_SIZE_BUTTON);
 
 	gtk_list_store_set (store, iter,
-			    COL_ACCOUNT_IMAGE, icon_name,
+			    COL_ACCOUNT_IMAGE, pixbuf,
 			    COL_ACCOUNT_TEXT, tp_account_get_display_name (account),
 			    COL_ACCOUNT_ENABLED, is_enabled,
 			    -1);
 
+	if (pixbuf != NULL)
+		g_object_unref (pixbuf);
+
 	/* set first connected account as active account */
 	if (priv->account_manually_set == FALSE &&
 	    priv->set_active_item == FALSE && is_enabled) {
@@ -786,7 +846,6 @@ account_chooser_filter_ready_cb (gboolean is_enabled,
 		gtk_combo_box_set_active_iter (combobox, iter);
 	}
 
-	g_object_unref (account);
 	filter_result_callback_data_free (fr_data);
 }
 
@@ -1049,3 +1108,11 @@ empathy_account_chooser_get_account (EmpathyAccountChooser *chooser)
 
 	return account;
 }
+
+TpAccountManager *
+empathy_account_chooser_get_account_manager (EmpathyAccountChooser *self)
+{
+	EmpathyAccountChooserPriv *priv = GET_PRIV (self);
+
+	return priv->manager;
+}
