@@ -31,6 +31,7 @@
 #include <folks/folks.h>
 #include <folks/folks-telepathy.h>
 
+#include <libempathy/empathy-camera-monitor.h>
 #include <libempathy/empathy-request-util.h>
 #include <libempathy/empathy-individual-manager.h>
 #include <libempathy/empathy-chatroom-manager.h>
@@ -46,6 +47,7 @@
 #include "empathy-ui-utils.h"
 #include "empathy-share-my-desktop.h"
 #include "empathy-linking-dialog.h"
+#include "empathy-call-utils.h"
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyIndividualMenu)
 
@@ -74,23 +76,28 @@ individual_menu_add_personas (GtkMenuShell *menu,
     EmpathyIndividualFeatureFlags features)
 {
   GtkWidget *item;
-  GList *personas, *l;
+  GeeSet *personas;
+  GeeIterator *iter;
   guint persona_count = 0;
+  gboolean c;
 
   g_return_if_fail (GTK_IS_MENU (menu));
   g_return_if_fail (FOLKS_IS_INDIVIDUAL (individual));
   g_return_if_fail (empathy_folks_individual_contains_contact (individual));
 
   personas = folks_individual_get_personas (individual);
+  /* we'll re-use this iterator throughout */
+  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
 
   /* Make sure we've got enough valid entries for these menu items to add
    * functionality */
-  for (l = personas; l != NULL; l = l->next)
+  while (gee_iterator_next (iter))
     {
-      if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
-        continue;
+      FolksPersona *persona = gee_iterator_get (iter);
+      if (empathy_folks_persona_is_interesting (persona))
+        persona_count++;
 
-      persona_count++;
+      g_clear_object (&persona);
     }
 
   /* return early if these entries would add nothing beyond the "quick" items */
@@ -103,21 +110,21 @@ individual_menu_add_personas (GtkMenuShell *menu,
   gtk_widget_show (item);
 
   personas = folks_individual_get_personas (individual);
-  for (l = personas; l != NULL; l = l->next)
+  for (c = gee_iterator_first (iter); c; c = gee_iterator_next (iter))
     {
       GtkWidget *image;
       GtkWidget *contact_item;
       GtkWidget *contact_submenu;
       TpContact *tp_contact;
       EmpathyContact *contact;
-      TpfPersona *persona = l->data;
+      TpfPersona *persona = gee_iterator_get (iter);
       gchar *label;
       FolksPersonaStore *store;
       const gchar *account;
       GtkWidget *action;
 
-      if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
-        continue;
+      if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (persona)))
+        goto while_finish;
 
       tp_contact = tpf_persona_get_contact (persona);
       contact = empathy_contact_dup_from_tp_contact (tp_contact);
@@ -149,6 +156,14 @@ individual_menu_add_personas (GtkMenuShell *menu,
           gtk_widget_show (action);
         }
 
+      /* SMS */
+      if (features & EMPATHY_INDIVIDUAL_FEATURE_SMS)
+        {
+          action = empathy_individual_sms_menu_item_new (NULL, contact);
+          gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
+          gtk_widget_show (action);
+        }
+
       if (features & EMPATHY_INDIVIDUAL_FEATURE_CALL)
         {
           /* Audio Call */
@@ -191,7 +206,12 @@ individual_menu_add_personas (GtkMenuShell *menu,
 
       g_free (label);
       g_object_unref (contact);
+
+while_finish:
+      g_clear_object (&persona);
     }
+
+  g_clear_object (&iter);
 }
 
 static void
@@ -238,6 +258,17 @@ constructed (GObject *object)
         }
     }
 
+  /* SMS */
+  if (features & EMPATHY_INDIVIDUAL_FEATURE_SMS)
+    {
+      item = empathy_individual_sms_menu_item_new (individual, NULL);
+      if (item != NULL)
+        {
+          gtk_menu_shell_append (shell, item);
+          gtk_widget_show (item);
+        }
+    }
+
   if (features & EMPATHY_INDIVIDUAL_FEATURE_CALL)
     {
       /* Audio Call */
@@ -499,7 +530,7 @@ empathy_individual_chat_menu_item_activated (GtkMenuItem *item,
 {
   g_return_if_fail (EMPATHY_IS_CONTACT (contact));
 
-  empathy_chat_with_contact (contact, gtk_get_current_event_time ());
+  empathy_chat_with_contact (contact, empathy_get_current_action_time ());
 }
 
 GtkWidget *
@@ -536,14 +567,62 @@ empathy_individual_chat_menu_item_new (FolksIndividual *individual,
   return item;
 }
 
+static void
+empathy_individual_sms_menu_item_activated (GtkMenuItem *item,
+  EmpathyContact *contact)
+{
+  g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+
+  empathy_sms_contact_id (
+      empathy_contact_get_account (contact),
+      empathy_contact_get_id (contact),
+      empathy_get_current_action_time ());
+}
+
+GtkWidget *
+empathy_individual_sms_menu_item_new (FolksIndividual *individual,
+    EmpathyContact *contact)
+{
+  GtkWidget *item;
+  GtkWidget *image;
+
+  g_return_val_if_fail ((FOLKS_IS_INDIVIDUAL (individual) &&
+      empathy_folks_individual_contains_contact (individual)) ||
+      EMPATHY_IS_CONTACT (contact),
+      NULL);
+
+  item = gtk_image_menu_item_new_with_mnemonic (_("_SMS"));
+  image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_SMS,
+      GTK_ICON_SIZE_MENU);
+  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+  gtk_widget_show (image);
+
+  if (contact != NULL)
+    {
+      menu_item_set_contact (item, contact,
+          G_CALLBACK (empathy_individual_sms_menu_item_activated),
+          EMPATHY_ACTION_SMS);
+    }
+  else
+    {
+      menu_item_set_first_contact (item, individual,
+          G_CALLBACK (empathy_individual_sms_menu_item_activated),
+          EMPATHY_ACTION_SMS);
+    }
+
+  return item;
+}
+
 static void
 empathy_individual_audio_call_menu_item_activated (GtkMenuItem *item,
   EmpathyContact *contact)
 {
   g_return_if_fail (EMPATHY_IS_CONTACT (contact));
 
-  empathy_call_new_with_streams (contact, TRUE, FALSE,
-      gtk_get_current_event_time ());
+  empathy_call_new_with_streams (empathy_contact_get_id (contact),
+      empathy_contact_get_account (contact),
+      TRUE, FALSE,
+      empathy_get_current_action_time ());
 }
 
 GtkWidget *
@@ -584,8 +663,10 @@ empathy_individual_video_call_menu_item_activated (GtkMenuItem *item,
 {
   g_return_if_fail (EMPATHY_IS_CONTACT (contact));
 
-  empathy_call_new_with_streams (contact, TRUE, TRUE,
-      gtk_get_current_event_time ());
+  empathy_call_new_with_streams (empathy_contact_get_id (contact),
+      empathy_contact_get_account (contact),
+      TRUE, TRUE,
+      empathy_get_current_action_time ());
 }
 
 GtkWidget *
@@ -594,6 +675,7 @@ empathy_individual_video_call_menu_item_new (FolksIndividual *individual,
 {
   GtkWidget *item;
   GtkWidget *image;
+  EmpathyCameraMonitor *monitor;
 
   g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual) ||
       EMPATHY_IS_CONTACT (contact),
@@ -618,6 +700,16 @@ empathy_individual_video_call_menu_item_new (FolksIndividual *individual,
           EMPATHY_ACTION_VIDEO_CALL);
     }
 
+  /* Only follow available cameras if the contact can do Video calls */
+  if (gtk_widget_get_sensitive (item))
+    {
+      monitor = empathy_camera_monitor_dup_singleton ();
+      g_object_set_data_full (G_OBJECT (item),
+          "monitor", monitor, g_object_unref);
+      g_object_bind_property (monitor, "available", item, "sensitive",
+          G_BINDING_SYNC_CREATE);
+    }
+
   return item;
 }
 
@@ -916,7 +1008,6 @@ room_sub_menu_activate_cb (GtkWidget *item,
   EmpathyTpChat *chat;
   EmpathyChatroomManager *mgr;
   EmpathyContact *contact = NULL;
-  GList *personas, *l;
 
   chat = empathy_chatroom_get_tp_chat (data->chatroom);
   if (chat == NULL)
@@ -931,30 +1022,37 @@ room_sub_menu_activate_cb (GtkWidget *item,
     contact = g_object_ref (data->contact);
   else
     {
+      GeeSet *personas;
+      GeeIterator *iter;
+
       /* find the first of this Individual's contacts who can join this room */
       personas = folks_individual_get_personas (data->individual);
-      for (l = personas; l != NULL && contact == NULL; l = g_list_next (l))
+
+      iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+      while (gee_iterator_next (iter) && (contact == NULL))
         {
-          TpfPersona *persona = l->data;
+          TpfPersona *persona = gee_iterator_get (iter);
           TpContact *tp_contact;
           GList *rooms;
 
-          if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
-            continue;
+          if (empathy_folks_persona_is_interesting (FOLKS_PERSONA (persona)))
+            {
+              tp_contact = tpf_persona_get_contact (persona);
+              contact = empathy_contact_dup_from_tp_contact (tp_contact);
 
-          tp_contact = tpf_persona_get_contact (persona);
-          contact = empathy_contact_dup_from_tp_contact (tp_contact);
+              rooms = empathy_chatroom_manager_get_chatrooms (mgr,
+                  empathy_contact_get_account (contact));
 
-          rooms = empathy_chatroom_manager_get_chatrooms (mgr,
-              empathy_contact_get_account (contact));
+              if (g_list_find (rooms, data->chatroom) == NULL)
+                g_clear_object (&contact);
 
-          if (g_list_find (rooms, data->chatroom) == NULL)
-            tp_clear_object (&contact);
+              /* if contact != NULL here, we've found our match */
 
-          /* if contact != NULL here, we've found our match */
-
-          g_list_free (rooms);
+              g_list_free (rooms);
+            }
+          g_clear_object (&persona);
         }
+      g_clear_object (&iter);
     }
 
   g_object_unref (mgr);
@@ -1000,7 +1098,6 @@ empathy_individual_invite_menu_item_new (FolksIndividual *individual,
   GtkWidget *image;
   GtkWidget *room_item;
   EmpathyChatroomManager *mgr;
-  GList *personas;
   GList *rooms = NULL;
   GList *names = NULL;
   GList *l;
@@ -1030,27 +1127,33 @@ empathy_individual_invite_menu_item_new (FolksIndividual *individual,
     }
   else
     {
-      /* collect the rooms from amongst all accounts for this Individual */
+      GeeSet *personas;
+      GeeIterator *iter;
+
+      /* find the first of this Individual's contacts who can join this room */
       personas = folks_individual_get_personas (individual);
-      for (l = personas; l != NULL; l = g_list_next (l))
+      iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+      while (gee_iterator_next (iter))
         {
-          TpfPersona *persona = l->data;
+          TpfPersona *persona = gee_iterator_get (iter);
           GList *rooms_cur;
           TpContact *tp_contact;
           EmpathyContact *contact_cur;
 
-          if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
-            continue;
+          if (empathy_folks_persona_is_interesting (FOLKS_PERSONA (persona)))
+            {
+              tp_contact = tpf_persona_get_contact (persona);
+              contact_cur = empathy_contact_dup_from_tp_contact (tp_contact);
 
-          tp_contact = tpf_persona_get_contact (persona);
-          contact_cur = empathy_contact_dup_from_tp_contact (tp_contact);
+              rooms_cur = empathy_chatroom_manager_get_chatrooms (mgr,
+                  empathy_contact_get_account (contact_cur));
+              rooms = g_list_concat (rooms, rooms_cur);
 
-          rooms_cur = empathy_chatroom_manager_get_chatrooms (mgr,
-              empathy_contact_get_account (contact_cur));
-          rooms = g_list_concat (rooms, rooms_cur);
-
-          g_object_unref (contact_cur);
+              g_object_unref (contact_cur);
+            }
+          g_clear_object (&persona);
         }
+      g_clear_object (&iter);
     }
 
   /* alphabetize the rooms */
