@@ -28,7 +28,10 @@
 
 #include <telepathy-glib/interfaces.h>
 
+#include <telepathy-yell/telepathy-yell.h>
+
 #include <libempathy/empathy-tp-contact-factory.h>
+#include <libempathy/empathy-camera-monitor.h>
 #include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-request-util.h>
@@ -41,6 +44,7 @@
 
 #include "empathy-new-call-dialog.h"
 #include "empathy-account-chooser.h"
+#include "empathy-call-utils.h"
 
 static EmpathyNewCallDialog *dialog_singleton = NULL;
 
@@ -56,27 +60,14 @@ typedef struct {
 
 struct _EmpathyNewCallDialogPriv {
   GtkWidget *check_video;
+
+  EmpathyCameraMonitor *monitor;
 };
 
 #define GET_PRIV(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), EMPATHY_TYPE_NEW_CALL_DIALOG, \
     EmpathyNewCallDialogPriv))
 
-static void
-create_media_channel_cb (GObject *source,
-    GAsyncResult *result,
-    gpointer user_data)
-{
-  GError *error = NULL;
-
-  if (!tp_account_channel_request_create_channel_finish (
-        TP_ACCOUNT_CHANNEL_REQUEST (source), result, &error))
-    {
-      DEBUG ("Failed to create media channel: %s", error->message);
-      g_error_free (error);
-    }
-}
-
 /**
  * SECTION:empathy-new-call-dialog
  * @title: EmpathyNewCallDialog
@@ -87,34 +78,6 @@ create_media_channel_cb (GObject *source,
  * to be started with any contact on any enabled account.
  */
 
-static void
-call_contact (TpAccount *account,
-    const gchar *contact_id,
-    gboolean video,
-    gint64 timestamp)
-{
-  GHashTable *request;
-  TpAccountChannelRequest *req;
-
-  request = tp_asv_new (
-      TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING,
-        TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
-      TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT, TP_HANDLE_TYPE_CONTACT,
-      TP_PROP_CHANNEL_TARGET_ID, G_TYPE_STRING, contact_id,
-      TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_AUDIO, G_TYPE_BOOLEAN,
-        TRUE,
-      TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_VIDEO, G_TYPE_BOOLEAN,
-        video,
-      NULL);
-
-  req = tp_account_channel_request_new (account, request, timestamp);
-
-  tp_account_channel_request_create_channel_async (req, EMPATHY_AV_BUS_NAME,
-      NULL, create_media_channel_cb, NULL);
-
-  g_object_unref (req);
-}
-
 static void
 empathy_new_call_dialog_response (GtkDialog *dialog, int response_id)
 {
@@ -134,7 +97,8 @@ empathy_new_call_dialog_response (GtkDialog *dialog, int response_id)
    * we return from this function. */
   video = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_video));
 
-  call_contact (account, contact_id, video,
+  empathy_call_new_with_streams (contact_id,
+      account, TRUE, video,
       empathy_get_current_action_time ());
 
 out:
@@ -169,7 +133,8 @@ conn_prepared_cb (GObject *conn,
 
       chan_type = tp_asv_get_string (fixed, TP_PROP_CHANNEL_CHANNEL_TYPE);
 
-      if (tp_strdiff (chan_type, TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA))
+      if (tp_strdiff (chan_type, TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA)
+          && tp_strdiff (chan_type, TPY_IFACE_CHANNEL_TYPE_CALL))
         continue;
 
       if (tp_asv_get_uint32 (fixed, TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, NULL) !=
@@ -217,6 +182,16 @@ empathy_new_call_dialog_account_filter (EmpathyContactSelectorDialog *dialog,
   tp_proxy_prepare_async (connection, features, conn_prepared_cb, cb_data);
 }
 
+static void
+empathy_new_call_dialog_dispose (GObject *object)
+{
+  EmpathyNewCallDialogPriv *priv = GET_PRIV (object);
+
+  tp_clear_object (&priv->monitor);
+
+  G_OBJECT_CLASS (empathy_new_call_dialog_parent_class)->dispose (object);
+}
+
 static GObject *
 empathy_new_call_dialog_constructor (GType type,
     guint n_props,
@@ -250,8 +225,13 @@ empathy_new_call_dialog_init (EmpathyNewCallDialog *dialog)
   EmpathyNewCallDialogPriv *priv = GET_PRIV (dialog);
   GtkWidget *image;
 
+  priv->monitor = empathy_camera_monitor_dup_singleton ();
+
   /* add video toggle */
   priv->check_video = gtk_check_button_new_with_mnemonic (_("Send _Video"));
+  g_object_bind_property (priv->monitor, "available",
+      priv->check_video, "sensitive",
+      G_BINDING_SYNC_CREATE);
 
   gtk_box_pack_end (GTK_BOX (parent->vbox), priv->check_video,
       FALSE, TRUE, 0);
@@ -287,6 +267,7 @@ empathy_new_call_dialog_class_init (
   g_type_class_add_private (class, sizeof (EmpathyNewCallDialogPriv));
 
   object_class->constructor = empathy_new_call_dialog_constructor;
+  object_class->dispose = empathy_new_call_dialog_dispose;
 
   dialog_class->response = empathy_new_call_dialog_response;
 
