@@ -44,12 +44,12 @@
 
 #include "empathy-ui-utils.h"
 #include "empathy-images.h"
+#include "empathy-live-search.h"
 #include "empathy-smiley-manager.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include <libempathy/empathy-debug.h>
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-ft-factory.h>
 
 void
@@ -231,8 +231,8 @@ empathy_icon_name_for_individual (FolksIndividual *individual)
 	TpConnectionPresenceType presence;
 
 	folks_presence =
-	    folks_presence_owner_get_presence_type (
-	        FOLKS_PRESENCE_OWNER (individual));
+	    folks_presence_details_get_presence_type (
+	        FOLKS_PRESENCE_DETAILS (individual));
 	presence = empathy_folks_presence_type_to_tp (folks_presence);
 
 	return empathy_icon_name_for_presence (presence);
@@ -406,11 +406,10 @@ empathy_avatar_pixbuf_roundify (GdkPixbuf *pixbuf)
 static gboolean
 empathy_gdk_pixbuf_is_opaque (GdkPixbuf *pixbuf)
 {
-	gint width, height, rowstride, i;
+	gint height, rowstride, i;
 	guchar *pixels;
 	guchar *row;
 
-	width = gdk_pixbuf_get_width (pixbuf);
 	height = gdk_pixbuf_get_height (pixbuf);
 	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 	pixels = gdk_pixbuf_get_pixels (pixbuf);
@@ -633,7 +632,7 @@ empathy_pixbuf_avatar_from_individual_scaled_async (
 			empathy_pixbuf_avatar_from_individual_scaled_async);
 
 	avatar_file =
-		folks_avatar_owner_get_avatar (FOLKS_AVATAR_OWNER (individual));
+		folks_avatar_details_get_avatar (FOLKS_AVATAR_DETAILS (individual));
 	if (avatar_file == NULL)
 		goto out;
 
@@ -1534,56 +1533,6 @@ empathy_text_iter_backward_search (const GtkTextIter   *iter,
 	return retval;
 }
 
-gboolean
-empathy_window_get_is_visible (GtkWindow *window)
-{
-	GdkWindowState  state;
-	GdkWindow      *gdk_window;
-
-	g_return_val_if_fail (GTK_IS_WINDOW (window), FALSE);
-
-	gdk_window = gtk_widget_get_window (GTK_WIDGET (window));
-	if (!gdk_window) {
-		return FALSE;
-	}
-
-	state = gdk_window_get_state (gdk_window);
-	if (state & (GDK_WINDOW_STATE_WITHDRAWN | GDK_WINDOW_STATE_ICONIFIED)) {
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
-void
-empathy_window_iconify (GtkWindow *window, GtkStatusIcon *status_icon)
-{
-	GdkRectangle  icon_location;
-	gulong        data[4];
-	Display      *dpy;
-	GdkWindow    *gdk_window;
-
-	gtk_status_icon_get_geometry (status_icon, NULL, &icon_location, NULL);
-	gdk_window = gtk_widget_get_window (GTK_WIDGET (window));
-	dpy = GDK_WINDOW_XDISPLAY (gdk_window);
-
-	data[0] = icon_location.x;
-	data[1] = icon_location.y;
-	data[2] = icon_location.width;
-	data[3] = icon_location.height;
-
-	XChangeProperty (dpy,
-			 GDK_WINDOW_XID (gdk_window),
-			 gdk_x11_get_xatom_by_name_for_display (
-				gdk_window_get_display (gdk_window),
-			 "_NET_WM_ICON_GEOMETRY"),
-			 XA_CARDINAL, 32, PropModeReplace,
-			 (guchar *)&data, 4);
-
-	gtk_window_set_skip_taskbar_hint (window, TRUE);
-	gtk_window_iconify (window);
-}
-
 /* Takes care of moving the window to the current workspace. */
 void
 empathy_window_present_with_time (GtkWindow *window,
@@ -1615,9 +1564,6 @@ empathy_window_present_with_time (GtkWindow *window,
 		gtk_window_present (window);
 	else
 		gtk_window_present_with_time (window, timestamp);
-
-	gtk_window_set_skip_taskbar_hint (window, FALSE);
-	gtk_window_deiconify (window);
 }
 
 void
@@ -1839,11 +1785,66 @@ file_manager_receive_file_response_cb (GtkDialog *dialog,
 	GFile *file;
 
 	if (response == GTK_RESPONSE_OK) {
-		factory = empathy_ft_factory_dup_singleton ();
-		file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (dialog));
+		GFile *parent;
+		GFileInfo *info;
+		guint64 free_space, file_size;
+		GError *error = NULL;
 
-		empathy_ft_factory_set_destination_for_incoming_handler
-			(factory, handler, file);
+		file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (dialog));
+		parent = g_file_get_parent (file);
+		info = g_file_query_filesystem_info (parent,
+				G_FILE_ATTRIBUTE_FILESYSTEM_FREE,
+				NULL, &error);
+
+		g_object_unref (parent);
+
+		if (error != NULL) {
+			g_warning ("Error: %s", error->message);
+
+			g_object_unref (file);
+			return;
+		}
+
+		free_space = g_file_info_get_attribute_uint64 (info,
+				G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
+		file_size = empathy_ft_handler_get_total_bytes (handler);
+
+		g_object_unref (info);
+
+		if (file_size > free_space) {
+			GtkWidget *message = gtk_message_dialog_new (
+				GTK_WINDOW (dialog),
+				GTK_DIALOG_MODAL,
+				GTK_MESSAGE_ERROR,
+				GTK_BUTTONS_CLOSE,
+				_("Insufficient free space to save file"));
+			char *file_size_str, *free_space_str;
+
+			file_size_str = g_format_size_for_display (file_size);
+			free_space_str = g_format_size_for_display (free_space);
+
+			gtk_message_dialog_format_secondary_text (
+				GTK_MESSAGE_DIALOG (message),
+				_("%s of free space are required to save this "
+				  "file, but only %s is available. Please "
+				  "choose another location."),
+				file_size_str, free_space_str);
+
+			gtk_dialog_run (GTK_DIALOG (message));
+
+			g_free (file_size_str);
+			g_free (free_space_str);
+			gtk_widget_destroy (message);
+
+			g_object_unref (file);
+
+			return;
+		}
+
+		factory = empathy_ft_factory_dup_singleton ();
+
+		empathy_ft_factory_set_destination_for_incoming_handler (
+				factory, handler, file);
 
 		g_object_unref (factory);
 		g_object_unref (file);
@@ -1941,3 +1942,72 @@ empathy_context_menu_new (GtkWidget *attach_to)
 
 	return menu;
 }
+
+gint64
+empathy_get_current_action_time (void)
+{
+  return (tp_user_action_time_from_x11 (gtk_get_current_event_time ()));
+}
+
+/* @words = empathy_live_search_strip_utf8_string (@text);
+ *
+ * User has to pass both so we don't have to compute @words ourself each time
+ * this function is called. */
+gboolean
+empathy_individual_match_string (FolksIndividual *individual,
+    const char *text,
+    GPtrArray *words)
+{
+  const gchar *str;
+  GeeSet *personas;
+  GeeIterator *iter;
+  gboolean retval = FALSE;
+
+  /* check alias name */
+  str = folks_alias_details_get_alias (FOLKS_ALIAS_DETAILS (individual));
+
+  if (empathy_live_search_match_words (str, words))
+    return TRUE;
+
+  personas = folks_individual_get_personas (individual);
+
+  /* check contact id, remove the @server.com part */
+  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+  while (retval == FALSE && gee_iterator_next (iter))
+    {
+      FolksPersona *persona = gee_iterator_get (iter);
+      const gchar *p;
+
+      if (empathy_folks_persona_is_interesting (persona))
+        {
+          str = folks_persona_get_display_id (persona);
+
+          /* Accept the persona if @text is a full prefix of his ID; that allows
+           * user to find, say, a jabber contact by typing his JID. */
+          if (g_str_has_prefix (str, text))
+            {
+              retval = TRUE;
+            }
+          else
+            {
+              gchar *dup_str = NULL;
+              gboolean visible;
+
+              p = strstr (str, "@");
+              if (p != NULL)
+                str = dup_str = g_strndup (str, p - str);
+
+              visible = empathy_live_search_match_words (str, words);
+              g_free (dup_str);
+              if (visible)
+                retval = TRUE;
+            }
+        }
+      g_clear_object (&persona);
+    }
+  g_clear_object (&iter);
+
+  /* FIXME: Add more rules here, we could check phone numbers in
+   * contact's vCard for example. */
+  return retval;
+}
