@@ -40,6 +40,7 @@
 #endif
 
 #include "empathy-contact.h"
+#include "empathy-camera-monitor.h"
 #include "empathy-individual-manager.h"
 #include "empathy-utils.h"
 #include "empathy-enum-types.h"
@@ -56,6 +57,7 @@ typedef struct {
   FolksPersona *persona;
   gchar *id;
   gchar *alias;
+  gchar *logged_alias;
   EmpathyAvatar *avatar;
   TpConnectionPresenceType presence;
   guint handle;
@@ -109,6 +111,7 @@ enum
   PROP_PERSONA,
   PROP_ID,
   PROP_ALIAS,
+  PROP_LOGGED_ALIAS,
   PROP_AVATAR,
   PROP_PRESENCE,
   PROP_PRESENCE_MESSAGE,
@@ -317,6 +320,15 @@ empathy_contact_class_init (EmpathyContactClass *class)
         NULL,
         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (object_class,
+      PROP_LOGGED_ALIAS,
+      g_param_spec_string ("logged-alias",
+        "Logged alias",
+        "The alias the user had when a message was logged, "
+        "only set when using empathy_contact_from_tpl_contact()",
+        NULL,
+        G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   g_object_class_install_property (object_class,
       PROP_AVATAR,
       g_param_spec_boxed ("avatar",
@@ -557,6 +569,9 @@ contact_get_property (GObject *object,
       case PROP_ALIAS:
         g_value_set_string (value, empathy_contact_get_alias (contact));
         break;
+      case PROP_LOGGED_ALIAS:
+        g_value_set_string (value, empathy_contact_get_logged_alias (contact));
+        break;
       case PROP_AVATAR:
         g_value_set_boxed (value, empathy_contact_get_avatar (contact));
         break;
@@ -608,6 +623,10 @@ contact_set_property (GObject *object,
       case PROP_ALIAS:
         empathy_contact_set_alias (contact, g_value_get_string (value));
         break;
+      case PROP_LOGGED_ALIAS:
+        g_assert (priv->logged_alias == NULL);
+        priv->logged_alias = g_value_dup_string (value);
+        break;
       case PROP_PRESENCE:
         empathy_contact_set_presence (contact, g_value_get_uint (value));
         break;
@@ -665,12 +684,15 @@ contact_is_tpl_entity (gpointer key,
 {
   EmpathyContact *contact = value;
   FindContactData *data = user_data;
+  TpAccount *account = empathy_contact_get_account (contact);
+  const gchar *path = NULL;
+
+  if (account != NULL)
+    path = tp_proxy_get_object_path (account);
 
   return !tp_strdiff (empathy_contact_get_id (contact),
               tpl_entity_get_identifier (data->entity)) &&
-         !tp_strdiff (tp_proxy_get_object_path (data->account),
-              tp_proxy_get_object_path (
-                  empathy_contact_get_account (contact)));
+         !tp_strdiff (tp_proxy_get_object_path (data->account), path);
 }
 
 EmpathyContact *
@@ -696,20 +718,10 @@ empathy_contact_from_tpl_contact (TpAccount *account,
 
   if (existing_contact != NULL)
     {
-      EmpathyContactPriv *priv;
-
       retval = g_object_new (EMPATHY_TYPE_CONTACT,
           "tp-contact", empathy_contact_get_tp_contact (existing_contact),
+          "logged-alias", tpl_entity_get_alias (tpl_entity),
           NULL);
-
-      priv = GET_PRIV (retval);
-
-      /* contact_set_property() calls empathy_contact_set_alias(), which
-       * tries to set the alias on the FolksPersona, but we don't want to
-       * do that when creating an EmpathyContact from a TplEntity. So just
-       * set priv->alias instead of passing it to g_object_new() instead. */
-      g_free (priv->alias);
-      priv->alias = g_strdup (tpl_entity_get_alias (tpl_entity));
     }
   else
     {
@@ -778,6 +790,21 @@ empathy_contact_get_alias (EmpathyContact *contact)
     return empathy_contact_get_id (contact);
 }
 
+const gchar *
+empathy_contact_get_logged_alias (EmpathyContact *contact)
+{
+  EmpathyContactPriv *priv;
+
+  g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+  priv = GET_PRIV (contact);
+
+  if (priv->logged_alias != NULL)
+    return priv->logged_alias;
+  else
+    return empathy_contact_get_alias (contact);
+}
+
 void
 empathy_contact_set_alias (EmpathyContact *contact,
                           const gchar *alias)
@@ -1625,7 +1652,7 @@ geocode_cb (GObject *source,
   g_object_notify ((GObject *) contact, "location");
 
 out:
-  tp_clear_pointer (&result, g_hash_table_unref);
+  tp_clear_pointer (&resolved, g_hash_table_unref);
   g_object_unref (contact);
 }
 
@@ -1645,6 +1672,8 @@ update_geocode (EmpathyContact *contact)
     return;
 
   geocode = geocode_object_new_for_params (location);
+  if (geocode == NULL)
+    return;
 
   geocode_object_resolve_async (geocode, NULL, geocode_cb,
       g_object_ref (contact));
