@@ -1226,7 +1226,7 @@ debug_window_save_clicked_cb (GtkToolButton *tool_button,
       GTK_FILE_CHOOSER (file_chooser), TRUE);
 
   gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_chooser),
-      g_get_tmp_dir ());
+      g_get_home_dir ());
 
   name = get_active_service_name (debug_window);
 
@@ -1407,6 +1407,8 @@ am_prepared_cb (GObject *am,
   gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), TRUE);
   gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar),
       GTK_ICON_SIZE_SMALL_TOOLBAR);
+  gtk_style_context_add_class (gtk_widget_get_style_context (toolbar),
+			       GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
   gtk_widget_show (toolbar);
 
   gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, FALSE, 0);
