@@ -204,18 +204,15 @@ main_window_flash_foreach (GtkTreeModel *model,
 	GdkPixbuf        *pixbuf = NULL;
 
 	gtk_tree_model_get (model, iter,
-			    EMPATHY_INDIVIDUAL_STORE_COL_INDIVIDUAL,
-				&individual,
+			    EMPATHY_INDIVIDUAL_STORE_COL_INDIVIDUAL, &individual,
 			    -1);
 
 	if (individual == NULL)
 		return FALSE;
 
 	contact = empathy_contact_dup_from_folks_individual (individual);
-	if (contact != data->event->contact) {
-		tp_clear_object (&contact);
-		return FALSE;
-	}
+	if (contact != data->event->contact)
+		goto out;
 
 	if (data->on) {
 		icon_name = data->event->icon_name;
@@ -244,6 +241,7 @@ main_window_flash_foreach (GtkTreeModel *model,
 		gtk_tree_path_free (parent_path);
 	}
 
+out:
 	g_object_unref (individual);
 	tp_clear_object (&contact);
 	tp_clear_object (&pixbuf);
@@ -433,12 +431,94 @@ main_window_auth_display (EmpathyMainWindow *window,
 	g_hash_table_insert (priv->auths, event, info_bar);
 }
 
+static void
+modify_event_count (GtkTreeModel *model,
+		    GtkTreeIter *iter,
+		    EmpathyEvent *event,
+		    gboolean increase)
+{
+	FolksIndividual *individual;
+	EmpathyContact *contact;
+	guint count;
+
+	gtk_tree_model_get (model, iter,
+			    EMPATHY_INDIVIDUAL_STORE_COL_INDIVIDUAL, &individual,
+			    EMPATHY_INDIVIDUAL_STORE_COL_EVENT_COUNT, &count,
+			    -1);
+
+	if (individual == NULL)
+		return;
+
+	increase ? count++ : count--;
+
+	contact = empathy_contact_dup_from_folks_individual (individual);
+	if (contact == event->contact) {
+		gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+				    EMPATHY_INDIVIDUAL_STORE_COL_EVENT_COUNT, count, -1);
+	}
+
+	tp_clear_object (&contact);
+	g_object_unref (individual);
+}
+
+static gboolean
+increase_event_count_foreach (GtkTreeModel *model,
+			      GtkTreePath *path,
+			      GtkTreeIter *iter,
+			      gpointer user_data)
+{
+	EmpathyEvent *event = user_data;
+
+	modify_event_count (model, iter, event, TRUE);
+
+	return FALSE;
+}
+
+static void
+increase_event_count (EmpathyMainWindow *self,
+		      EmpathyEvent *event)
+{
+	EmpathyMainWindowPriv *priv = GET_PRIV (self);
+	GtkTreeModel *model;
+
+	model = GTK_TREE_MODEL (priv->individual_store);
+
+	gtk_tree_model_foreach (model, increase_event_count_foreach, event);
+}
+
+static gboolean
+decrease_event_count_foreach (GtkTreeModel *model,
+			      GtkTreePath *path,
+			      GtkTreeIter *iter,
+			      gpointer user_data)
+{
+	EmpathyEvent *event = user_data;
+
+	modify_event_count (model, iter, event, FALSE);
+
+	return FALSE;
+}
+
+static void
+decrease_event_count (EmpathyMainWindow *self,
+		      EmpathyEvent *event)
+{
+	EmpathyMainWindowPriv *priv = GET_PRIV (self);
+	GtkTreeModel *model;
+
+	model = GTK_TREE_MODEL (priv->individual_store);
+
+	gtk_tree_model_foreach (model, decrease_event_count_foreach, event);
+}
+
 static void
 main_window_event_added_cb (EmpathyEventManager *manager,
 			    EmpathyEvent        *event,
 			    EmpathyMainWindow   *window)
 {
 	if (event->contact) {
+		increase_event_count (window, event);
+
 		main_window_flash_start (window);
 	} else if (event->type == EMPATHY_EVENT_TYPE_AUTH) {
 		main_window_auth_display (window, event);
@@ -462,6 +542,8 @@ main_window_event_removed_cb (EmpathyEventManager *manager,
 		return;
 	}
 
+	decrease_event_count (window, event);
+
 	data.on = FALSE;
 	data.event = event;
 	data.window = window;
@@ -817,20 +899,17 @@ main_window_balance_activate_cb (GtkAction         *action,
 
 static void
 main_window_balance_update_balance (GtkAction   *action,
-				    GValueArray *balance)
+				    TpConnection *conn)
 {
 	TpAccount *account = g_object_get_data (G_OBJECT (action), "account");
 	GtkWidget *label;
 	int amount = 0;
 	guint scale = G_MAXINT32;
-	const char *currency = "";
+	const gchar *currency = "";
 	char *money, *str;
 
-	if (balance != NULL)
-		tp_value_array_unpack (balance, 3,
-			&amount,
-			&scale,
-			&currency);
+	if (!tp_connection_get_balance (conn, &amount, &scale, &currency))
+		return;
 
 	if (amount == 0 &&
 	    scale == G_MAXINT32 &&
@@ -860,42 +939,14 @@ main_window_balance_update_balance (GtkAction   *action,
 	g_free (money);
 }
 
-static void
-main_window_setup_balance_got_balance_props (TpProxy      *conn,
-					     GHashTable   *props,
-					     const GError *in_error,
-					     gpointer      user_data,
-					     GObject      *action)
-{
-	GValueArray *balance = NULL;
-	const char *uri;
-
-	if (in_error != NULL) {
-		DEBUG ("Failed to get account balance properties: %s",
-			in_error->message);
-		goto finally;
-	}
-
-	balance = tp_asv_get_boxed (props, "AccountBalance",
-		TP_STRUCT_TYPE_CURRENCY_AMOUNT);
-	uri = tp_asv_get_string (props, "ManageCreditURI");
-
-	g_object_set_data_full (action, "manage-credit-uri",
-		g_strdup (uri), g_free);
-	gtk_action_set_sensitive (GTK_ACTION (action), !tp_str_empty (uri));
-
-finally:
-	main_window_balance_update_balance (GTK_ACTION (action), balance);
-}
-
 static void
 main_window_balance_changed_cb (TpConnection      *conn,
-				const GValueArray *balance,
-				gpointer           user_data,
-				GObject           *action)
+				guint balance,
+				guint scale,
+				const gchar *currency,
+				GtkAction *action)
 {
-	main_window_balance_update_balance (GTK_ACTION (action),
-		(GValueArray *) balance);
+	main_window_balance_update_balance (action, conn);
 }
 
 static GtkAction *
@@ -1012,15 +1063,17 @@ main_window_setup_balance_create_widget (EmpathyMainWindow *window,
 }
 
 static void
-main_window_setup_balance_conn_ready (GObject      *conn,
+main_window_setup_balance_conn_ready (GObject      *source,
 				      GAsyncResult *result,
 				      gpointer      user_data)
 {
 	EmpathyMainWindow *window = user_data;
 	EmpathyMainWindowPriv *priv = GET_PRIV (window);
-	TpAccount *account = g_object_get_data (conn, "account");
+	TpConnection *conn = TP_CONNECTION (source);
+	TpAccount *account = g_object_get_data (source, "account");
 	GtkAction *action;
 	GError *error = NULL;
+	const gchar *uri;
 
 	if (!tp_proxy_prepare_finish (conn, result, &error)) {
 		DEBUG ("Failed to prepare connection: %s", error->message);
@@ -1029,10 +1082,8 @@ main_window_setup_balance_conn_ready (GObject      *conn,
 		return;
 	}
 
-	if (!tp_proxy_has_interface_by_id (conn,
-			TP_IFACE_QUARK_CONNECTION_INTERFACE_BALANCE)) {
+	if (!tp_proxy_is_prepared (conn, TP_CONNECTION_FEATURE_BALANCE))
 		return;
-	}
 
 	DEBUG ("Setting up balance for acct: %s",
 		tp_account_get_display_name (account));
@@ -1048,15 +1099,17 @@ main_window_setup_balance_conn_ready (GObject      *conn,
 	/* create the display widget */
 	main_window_setup_balance_create_widget (window, action);
 
-	/* request the current balance and monitor for any changes */
-	tp_cli_dbus_properties_call_get_all (conn, -1,
-		TP_IFACE_CONNECTION_INTERFACE_BALANCE,
-		main_window_setup_balance_got_balance_props,
-		window, NULL, G_OBJECT (action));
+	/* check the current balance and monitor for any changes */
+	uri = tp_connection_get_balance_uri (conn);
 
-	tp_cli_connection_interface_balance_connect_to_balance_changed (
-		TP_CONNECTION (conn), main_window_balance_changed_cb,
-		window, NULL, G_OBJECT (action), NULL);
+	g_object_set_data_full (G_OBJECT (action), "manage-credit-uri",
+		g_strdup (uri), g_free);
+	gtk_action_set_sensitive (GTK_ACTION (action), !tp_str_empty (uri));
+
+	main_window_balance_update_balance (GTK_ACTION (action), conn);
+
+	g_signal_connect (conn, "balance-changed",
+		G_CALLBACK (main_window_balance_changed_cb), action);
 }
 
 static void
@@ -1064,6 +1117,7 @@ main_window_setup_balance (EmpathyMainWindow *window,
 			   TpAccount         *account)
 {
 	TpConnection *conn = tp_account_get_connection (account);
+	GQuark features[] = { TP_CONNECTION_FEATURE_BALANCE, 0 };
 
 	if (conn == NULL)
 		return;
@@ -1071,10 +1125,54 @@ main_window_setup_balance (EmpathyMainWindow *window,
 	/* need to prepare the connection:
 	 * store the account on the connection */
 	g_object_set_data (G_OBJECT (conn), "account", account);
-	tp_proxy_prepare_async (conn, NULL,
+	tp_proxy_prepare_async (conn, features,
 		main_window_setup_balance_conn_ready, window);
 }
 
+static void
+main_window_remove_balance_action (EmpathyMainWindow *window,
+				   TpAccount         *account)
+{
+	EmpathyMainWindowPriv *priv = GET_PRIV (window);
+	GtkAction *action;
+	char *name;
+	GList *a;
+
+	if (priv->balance_action_group == NULL)
+		return;
+
+	name = main_window_account_to_action_name (account);
+
+	action = gtk_action_group_get_action (
+		priv->balance_action_group, name);
+
+	if (action != NULL) {
+		guint merge_id;
+
+		DEBUG ("Removing action");
+
+		merge_id = GPOINTER_TO_UINT (g_object_get_data (
+			G_OBJECT (action),
+			"merge-id"));
+
+		gtk_ui_manager_remove_ui (priv->ui_manager,
+			merge_id);
+		gtk_action_group_remove_action (
+			priv->balance_action_group, action);
+	}
+
+	g_free (name);
+
+	a = gtk_action_group_list_actions (
+		priv->balance_action_group);
+
+	gtk_action_set_visible (
+		priv->view_balance_show_in_roster,
+		g_list_length (a) > 0);
+
+	g_list_free (a);
+}
+
 static void
 main_window_connection_changed_cb (TpAccount  *account,
 				   guint       old_status,
@@ -1098,42 +1196,7 @@ main_window_connection_changed_cb (TpAccount  *account,
 				    EMPATHY_SOUND_ACCOUNT_DISCONNECTED);
 
 		/* remove balance action if required */
-		if (priv->balance_action_group != NULL) {
-			GtkAction *action;
-			char *name;
-			GList *a;
-
-			name = main_window_account_to_action_name (account);
-
-			action = gtk_action_group_get_action (
-				priv->balance_action_group, name);
-
-			if (action != NULL) {
-				guint merge_id;
-
-				DEBUG ("Removing action");
-
-				merge_id = GPOINTER_TO_UINT (g_object_get_data (
-					G_OBJECT (action),
-					"merge-id"));
-
-				gtk_ui_manager_remove_ui (priv->ui_manager,
-					merge_id);
-				gtk_action_group_remove_action (
-					priv->balance_action_group, action);
-			}
-
-			g_free (name);
-
-			a = gtk_action_group_list_actions (
-				priv->balance_action_group);
-
-			gtk_action_set_visible (
-				priv->view_balance_show_in_roster,
-				g_list_length (a) > 0);
-
-			g_list_free (a);
-		}
+		main_window_remove_balance_action (window, account);
 	}
 
 	if (current == TP_CONNECTION_STATUS_CONNECTED) {
@@ -1226,14 +1289,10 @@ main_window_key_press_event_cb  (GtkWidget   *window,
 				 GdkEventKey *event,
 				 gpointer     user_data)
 {
-	EmpathyChatManager *chat_manager;
-
 	if (event->keyval == GDK_KEY_T
 	    && event->state & GDK_SHIFT_MASK
 	    && event->state & GDK_CONTROL_MASK) {
-		chat_manager = empathy_chat_manager_dup_singleton ();
-		empathy_chat_manager_undo_closed_chat (chat_manager);
-		g_object_unref (chat_manager);
+		empathy_chat_manager_call_undo_closed_chat ();
 	}
 	return FALSE;
 }
@@ -1584,7 +1643,7 @@ main_window_favorite_chatroom_join (EmpathyChatroom *chatroom)
 		join_fav_account_sig_ctx *ctx;
 
 		ctx = join_fav_account_sig_ctx_new (account, chatroom,
-			gtk_get_current_event_time ());
+			empathy_get_current_action_time ());
 
 		ctx->sig_id = g_signal_connect_data (account, "status-changed",
 			G_CALLBACK (account_status_changed_cb), ctx,
@@ -1595,7 +1654,7 @@ main_window_favorite_chatroom_join (EmpathyChatroom *chatroom)
 		return;
 	}
 
-	join_chatroom (chatroom, gtk_get_current_event_time ());
+	join_chatroom (chatroom, empathy_get_current_action_time ());
 }
 
 static void
@@ -1611,14 +1670,20 @@ main_window_favorite_chatroom_menu_add (EmpathyMainWindow *window,
 {
 	EmpathyMainWindowPriv *priv = GET_PRIV (window);
 	GtkWidget   *menu_item;
-	const gchar *name;
+	const gchar *name, *account_name;
+	gchar *label;
+
 
 	if (g_object_get_data (G_OBJECT (chatroom), "menu_item")) {
 		return;
 	}
 
 	name = empathy_chatroom_get_name (chatroom);
-	menu_item = gtk_menu_item_new_with_label (name);
+	account_name = tp_account_get_display_name (
+			empathy_chatroom_get_account (chatroom));
+	label = g_strdup_printf ("%s (%s)", name, account_name);
+	menu_item = gtk_menu_item_new_with_label (label);
+	g_free (label);
 	g_object_set_data (G_OBJECT (menu_item), "is_favorite",
 			GUINT_TO_POINTER (TRUE));
 
@@ -1917,6 +1982,9 @@ main_window_account_removed_cb (TpAccountManager  *manager,
 
 	/* remove errors if any */
 	main_window_remove_error (window, account);
+
+	/* remove the balance action if required */
+	main_window_remove_balance_action (window, account);
 }
 
 static void
